import { defu } from 'defu';
import { defineNuxtModule, createResolver, resolveModule, addComponent, addAutoImport } from '@nuxt/kit';

const module = defineNuxtModule({
  meta: {
    name: "@nuxtlabs/github-module",
    configKey: "github"
  },
  defaults: {
    owner: "",
    repo: "",
    token: void 0,
    branch: "main",
    api: "https://api.github.com",
    remarkPlugin: true,
    disableCache: false,
    releases: true,
    contributors: true,
    maxContributors: 100,
    parseContents: true
  },
  setup(options, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    const runtimeDir = resolve("./runtime");
    if (!options.owner) {
      if (options.repo && options.repo.includes("/")) {
        const [owner, repo] = options.repo.split("/");
        options.owner = owner;
        options.repo = repo;
      }
    }
    if (!nuxt.options.runtimeConfig.public) {
      nuxt.options.runtimeConfig.public = {};
    }
    const config = {
      api: options.api || process.env.GITHUB_OWNER,
      owner: options.owner || process.env.GITHUB_OWNER,
      branch: options.branch || process.env.GITHUB_BRANCH,
      repo: options.repo || process.env.GITHUB_REPO,
      token: options.token || process.env.GITHUB_TOKEN,
      disableCache: options.disableCache,
      parseContents: options.parseContents,
      maxContributors: options.maxContributors
    };
    nuxt.options.runtimeConfig.public.github = config;
    nuxt.options.runtimeConfig.github = config;
    if (options.remarkPlugin) {
      nuxt.hook("content:context", (context) => {
        context.markdown.remarkPlugins = context.markdown.remarkPlugins || {};
        context.markdown.remarkPlugins["remark-github"] = { repository: `${options.owner}/${options.repo}` };
      });
    }
    const nitroConfig = nuxt.options.nitro;
    nitroConfig.handlers = nitroConfig.handlers || [];
    nitroConfig.prerender = nitroConfig.prerender || {};
    nitroConfig.prerender.routes = nitroConfig.prerender.routes || [];
    nitroConfig.handlers.push({
      route: "/api/_github/repository",
      handler: resolveModule("./server/api/repository", { paths: runtimeDir })
    }, {
      route: "/api/_github/repository/:query",
      handler: resolveModule("./server/api/repository", { paths: runtimeDir })
    });
    nitroConfig.handlers.push({
      route: "/api/_github/readme",
      handler: resolveModule("./server/api/readme", { paths: runtimeDir })
    }, {
      route: "/api/_github/readme/:query",
      handler: resolveModule("./server/api/readme", { paths: runtimeDir })
    });
    addComponent({
      name: "GithubRepository",
      filePath: resolveModule("./components/GithubRepository", { paths: runtimeDir }),
      global: true
    });
    addComponent({
      name: "GithubLink",
      filePath: resolveModule("./components/GithubLink", { paths: runtimeDir }),
      global: true
    });
    addComponent({
      name: "GithubReadme",
      filePath: resolveModule("./components/GithubReadme", { paths: runtimeDir }),
      global: true
    });
    if (options.releases) {
      nitroConfig.handlers.push({
        route: "/api/_github/releases",
        handler: resolveModule("./server/api/releases/index", { paths: runtimeDir })
      }, {
        route: "/api/_github/releases/:query",
        handler: resolveModule("./server/api/releases/index", { paths: runtimeDir })
      });
      nitroConfig.prerender.routes.push("/api/_github/releases");
      addComponent({
        name: "GithubReleases",
        filePath: resolveModule("./components/GithubReleases", { paths: runtimeDir }),
        global: true
      });
      addComponent({
        name: "GithubLastRelease",
        filePath: resolveModule("./components/GithubLastRelease", { paths: runtimeDir }),
        global: true
      });
      addComponent({
        name: "GithubRelease",
        filePath: resolveModule("./components/GithubRelease", { paths: runtimeDir }),
        global: true
      });
    }
    if (options.contributors) {
      nitroConfig.handlers.push({
        route: "/api/_github/contributors",
        handler: resolveModule("./server/api/contributors", { paths: runtimeDir })
      }, {
        route: "/api/_github/contributors/:query",
        handler: resolveModule("./server/api/contributors", { paths: runtimeDir })
      });
      nitroConfig.prerender.routes.push("/api/_github/contributors");
      nitroConfig.handlers.push({
        route: "/api/_github/contributors/file",
        handler: resolveModule("./server/api/contributors/file", { paths: runtimeDir })
      }, {
        route: "/api/_github/contributors/file/:query",
        handler: resolveModule("./server/api/contributors/file", { paths: runtimeDir })
      });
      addComponent({
        name: "GithubContributors",
        filePath: resolveModule("./components/GithubContributors", { paths: runtimeDir }),
        global: true
      });
      addComponent({
        name: "GithubFileContributors",
        filePath: resolveModule("./components/GithubFileContributors", { paths: runtimeDir }),
        global: true
      });
    }
    addAutoImport({
      name: "useGithub",
      from: resolveModule("./composables/useGithub", { paths: runtimeDir })
    });
    nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
      inline: [
        resolve("./runtime")
      ]
    });
  }
});

export { module as default };
