import { joinURL, withQuery } from "ufo";
import { graphql } from "@octokit/graphql";
import remarkGithub from "remark-github";
import { defu } from "defu";
import { parseContent } from "#content/server";
export function decodeParams(params = "") {
  const result = {};
  for (const param of params.split(",")) {
    const [key, ...value] = param.split("_");
    result[key] = value.join("_");
  }
  return result;
}
function isBot(user) {
  return user.login.includes("[bot]") || user.login.includes("-bot") || user.login.includes(".bot");
}
function normalizeRelease(release) {
  return {
    name: normalizeReleaseName(release?.name || release?.tag_name),
    tag_name: release?.tag_name,
    date: release?.published_at,
    body: release?.body,
    v: +normalizeReleaseName(release?.tag_name)?.substring(1, 2) || 0,
    url: release?.html_url,
    tarball: release?.tarball_url,
    zipball: release?.zipball_url,
    prerelease: release?.prerelease,
    reactions: release?.reactions,
    author: {
      name: release?.author?.login,
      url: release?.author?.html_url,
      avatar: release?.author?.avatar_url
    }
  };
}
function normalizeReleaseName(name) {
  if (!name) {
    return "";
  }
  name = name.replace("Release ", "");
  if (!name.match(/^[a-zA-Z]/)) {
    name = `v${name}`;
  }
  return name;
}
export function githubGraphqlQuery(query, options) {
  const gq = graphql.defaults({
    headers: {
      authorization: `token ${options.token}`
    }
  });
  return gq(query);
}
export const parseRelease = async (release, githubConfig) => {
  let parsedRelease;
  try {
    parsedRelease = {
      ...release,
      ...typeof parseContent === "function" && release?.body && release?.name ? await parseContent(`github:${release.name}.md`, release.body, {
        markdown: {
          remarkPlugins: {
            "remark-github": {
              instance: remarkGithub,
              repository: `${githubConfig.owner}/${githubConfig.repo}`
            }
          }
        }
      }) : {}
    };
  } catch (_err) {
    console.warn(`Cannot parse release ${release?.name} [${_err.response?.status || 500}]`);
    return;
  }
  return parsedRelease;
};
export function overrideConfig(config, query) {
  return (({ owner, repo, branch, api, token }) => ({ owner, repo, branch, api, token }))(defu(query, config));
}
export async function fetchRepository({ api, owner, repo, token }) {
  const url = `${api}/repos/${owner}/${repo}`;
  const repository = await $fetch(url, {
    headers: {
      Authorization: token ? `token ${token}` : void 0
    }
  }).catch((_) => {
    return {};
  });
  return repository;
}
export async function fetchRepositoryContributors({ max }, { api, owner, repo, token }) {
  let url = `${api}/repos/${owner}/${repo}/contributors`;
  url = withQuery(url, { max });
  const contributors = await $fetch(url, {
    headers: {
      Authorization: token ? `token ${token}` : void 0
    }
  }).catch((_) => {
    return [];
  });
  return contributors.map(({ avatar_url, login }) => ({ avatar_url, login }));
}
export async function fetchFileContributors({ source, max }, { owner, repo, branch, token }) {
  const data = await githubGraphqlQuery(
    `
  query {
    repository(owner: "${owner}", name: "${repo}") {
      object(expression: "${branch}") {
        ... on Commit {
          history(first: ${max}, path: "${source}") {
            nodes {
              authors(first: ${max}) {
                nodes {
                  user {
                    name
                    avatarUrl
                    login
                  }
                }
              }
            }
          }
        }
      }
    }
  }`,
    { token }
  ).catch((_) => {
  });
  if (!data?.repository?.object?.history?.nodes) {
    return [];
  }
  let users = data.repository.object.history.nodes.map((node) => node.authors.nodes).flat().map((node) => node.user).filter((user) => user && user.name).filter((user) => !isBot(user));
  users = users.reduce((unique, user) => unique.find((u) => u.login === user.login) ? unique : unique.concat(user), []);
  return users.map(({ avatarUrl, name, login }) => ({ avatar_url: avatarUrl, name, login }));
}
export async function fetchReleases(query, { api, repo, token, owner }) {
  const page = query?.page || 1;
  const perPage = query?.per_page || 100;
  const last = query?.last || false;
  const tag = query?.tag || false;
  let url = `${api}/repos/${owner}/${repo}/releases`;
  if (tag) {
    url = joinURL(url, "tags", tag);
  } else if (last) {
    url = joinURL(url, "latest");
  } else {
    url = withQuery(url, { per_page: perPage, page });
  }
  const rawReleases = await $fetch(url, {
    headers: {
      Authorization: token ? `token ${token}` : void 0
    }
  }).catch((_) => {
  });
  if (!rawReleases) {
    return last ? {} : [];
  }
  return last || tag ? normalizeRelease(rawReleases) : rawReleases.filter((r) => !r.draft).map(normalizeRelease);
}
export async function fetchReadme({ api, owner, repo, token }) {
  const url = `${api}/repos/${owner}/${repo}/readme`;
  const readme = await $fetch(url, {
    headers: {
      Authorization: token ? `token ${token}` : void 0
    }
  }).catch((_) => {
    return {};
  });
  return readme;
}
